/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1999 Spencer Kimball and Peter Mattis
 *
 * pikastrokeoptions.h
 * Copyright (C) 2003 Simon Budig
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_STROKE_OPTIONS_H__
#define __PIKA_STROKE_OPTIONS_H__


#include "pikafilloptions.h"


#define PIKA_TYPE_STROKE_OPTIONS            (pika_stroke_options_get_type ())
#define PIKA_STROKE_OPTIONS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_STROKE_OPTIONS, PikaStrokeOptions))
#define PIKA_STROKE_OPTIONS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_STROKE_OPTIONS, PikaStrokeOptionsClass))
#define PIKA_IS_STROKE_OPTIONS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_STROKE_OPTIONS))
#define PIKA_IS_STROKE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_STROKE_OPTIONS))
#define PIKA_STROKE_OPTIONS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_STROKE_OPTIONS, PikaStrokeOptionsClass))


typedef struct _PikaStrokeOptionsClass PikaStrokeOptionsClass;

struct _PikaStrokeOptions
{
  PikaFillOptions  parent_instance;
};

struct _PikaStrokeOptionsClass
{
  PikaFillOptionsClass  parent_class;

  void (* dash_info_changed) (PikaStrokeOptions *stroke_options,
                              PikaDashPreset     preset);
};


GType               pika_stroke_options_get_type             (void) G_GNUC_CONST;

PikaStrokeOptions * pika_stroke_options_new                  (Pika              *pika,
                                                              PikaContext       *context,
                                                              gboolean           use_context_color);

PikaStrokeMethod    pika_stroke_options_get_method           (PikaStrokeOptions *options);

gdouble             pika_stroke_options_get_width            (PikaStrokeOptions *options);
PikaUnit            pika_stroke_options_get_unit             (PikaStrokeOptions *options);
PikaCapStyle        pika_stroke_options_get_cap_style        (PikaStrokeOptions *options);
PikaJoinStyle       pika_stroke_options_get_join_style       (PikaStrokeOptions *options);
gdouble             pika_stroke_options_get_miter_limit      (PikaStrokeOptions *options);
gdouble             pika_stroke_options_get_dash_offset      (PikaStrokeOptions *options);
GArray            * pika_stroke_options_get_dash_info        (PikaStrokeOptions *options);

PikaPaintOptions  * pika_stroke_options_get_paint_options    (PikaStrokeOptions *options);
gboolean            pika_stroke_options_get_emulate_dynamics (PikaStrokeOptions *options);

void                pika_stroke_options_take_dash_pattern    (PikaStrokeOptions *options,
                                                              PikaDashPreset     preset,
                                                              GArray            *pattern);

void                pika_stroke_options_prepare              (PikaStrokeOptions *options,
                                                              PikaContext       *context,
                                                              PikaPaintOptions  *paint_options);
void                pika_stroke_options_finish               (PikaStrokeOptions *options);


#endif /* __PIKA_STROKE_OPTIONS_H__ */
